#ifndef __RESOURCE_cui_H__
#define __RESOURCE_cui_H__

#include <gio/gio.h>

G_GNUC_INTERNAL GResource *cui_get_resource (void);

G_GNUC_INTERNAL void cui_register_resource (void);
G_GNUC_INTERNAL void cui_unregister_resource (void);

#endif
