/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __PHOSH_GTK_MOUNTOPERATION_DBUS_H__
#define __PHOSH_GTK_MOUNTOPERATION_DBUS_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.Gtk.MountOperationHandler */

#define PHOSH_DBUS_TYPE_MOUNT_OPERATION_HANDLER (phosh_dbus_mount_operation_handler_get_type ())
#define PHOSH_DBUS_MOUNT_OPERATION_HANDLER(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_MOUNT_OPERATION_HANDLER, PhoshDBusMountOperationHandler))
#define PHOSH_DBUS_IS_MOUNT_OPERATION_HANDLER(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_MOUNT_OPERATION_HANDLER))
#define PHOSH_DBUS_MOUNT_OPERATION_HANDLER_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), PHOSH_DBUS_TYPE_MOUNT_OPERATION_HANDLER, PhoshDBusMountOperationHandlerIface))

struct _PhoshDBusMountOperationHandler;
typedef struct _PhoshDBusMountOperationHandler PhoshDBusMountOperationHandler;
typedef struct _PhoshDBusMountOperationHandlerIface PhoshDBusMountOperationHandlerIface;

struct _PhoshDBusMountOperationHandlerIface
{
  GTypeInterface parent_iface;

  gboolean (*handle_ask_password) (
    PhoshDBusMountOperationHandler *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_object_id,
    const gchar *arg_message,
    const gchar *arg_icon_name,
    const gchar *arg_default_user,
    const gchar *arg_default_domain,
    guint arg_flags);

  gboolean (*handle_ask_question) (
    PhoshDBusMountOperationHandler *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_object_id,
    const gchar *arg_message,
    const gchar *arg_icon_name,
    const gchar *const *arg_choices);

  gboolean (*handle_close) (
    PhoshDBusMountOperationHandler *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_show_processes) (
    PhoshDBusMountOperationHandler *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_object_id,
    const gchar *arg_message,
    const gchar *arg_icon_name,
    GVariant *arg_application_pids,
    const gchar *const *arg_choices);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusMountOperationHandler, g_object_unref)
#endif

GType phosh_dbus_mount_operation_handler_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *phosh_dbus_mount_operation_handler_interface_info (void);
guint phosh_dbus_mount_operation_handler_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void phosh_dbus_mount_operation_handler_complete_ask_password (
    PhoshDBusMountOperationHandler *object,
    GDBusMethodInvocation *invocation,
    guint response,
    GVariant *response_details);

void phosh_dbus_mount_operation_handler_complete_ask_question (
    PhoshDBusMountOperationHandler *object,
    GDBusMethodInvocation *invocation,
    guint response,
    GVariant *response_details);

void phosh_dbus_mount_operation_handler_complete_show_processes (
    PhoshDBusMountOperationHandler *object,
    GDBusMethodInvocation *invocation,
    guint response,
    GVariant *response_details);

void phosh_dbus_mount_operation_handler_complete_close (
    PhoshDBusMountOperationHandler *object,
    GDBusMethodInvocation *invocation);



/* D-Bus method calls: */
void phosh_dbus_mount_operation_handler_call_ask_password (
    PhoshDBusMountOperationHandler *proxy,
    const gchar *arg_object_id,
    const gchar *arg_message,
    const gchar *arg_icon_name,
    const gchar *arg_default_user,
    const gchar *arg_default_domain,
    guint arg_flags,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_mount_operation_handler_call_ask_password_finish (
    PhoshDBusMountOperationHandler *proxy,
    guint* out_response,
    GVariant ** out_response_details,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_mount_operation_handler_call_ask_password_sync (
    PhoshDBusMountOperationHandler *proxy,
    const gchar *arg_object_id,
    const gchar *arg_message,
    const gchar *arg_icon_name,
    const gchar *arg_default_user,
    const gchar *arg_default_domain,
    guint arg_flags,
    guint* out_response,
    GVariant ** out_response_details,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_mount_operation_handler_call_ask_question (
    PhoshDBusMountOperationHandler *proxy,
    const gchar *arg_object_id,
    const gchar *arg_message,
    const gchar *arg_icon_name,
    const gchar *const *arg_choices,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_mount_operation_handler_call_ask_question_finish (
    PhoshDBusMountOperationHandler *proxy,
    guint* out_response,
    GVariant ** out_response_details,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_mount_operation_handler_call_ask_question_sync (
    PhoshDBusMountOperationHandler *proxy,
    const gchar *arg_object_id,
    const gchar *arg_message,
    const gchar *arg_icon_name,
    const gchar *const *arg_choices,
    guint* out_response,
    GVariant ** out_response_details,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_mount_operation_handler_call_show_processes (
    PhoshDBusMountOperationHandler *proxy,
    const gchar *arg_object_id,
    const gchar *arg_message,
    const gchar *arg_icon_name,
    GVariant *arg_application_pids,
    const gchar *const *arg_choices,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_mount_operation_handler_call_show_processes_finish (
    PhoshDBusMountOperationHandler *proxy,
    guint* out_response,
    GVariant ** out_response_details,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_mount_operation_handler_call_show_processes_sync (
    PhoshDBusMountOperationHandler *proxy,
    const gchar *arg_object_id,
    const gchar *arg_message,
    const gchar *arg_icon_name,
    GVariant *arg_application_pids,
    const gchar *const *arg_choices,
    guint* out_response,
    GVariant ** out_response_details,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_mount_operation_handler_call_close (
    PhoshDBusMountOperationHandler *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_mount_operation_handler_call_close_finish (
    PhoshDBusMountOperationHandler *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_mount_operation_handler_call_close_sync (
    PhoshDBusMountOperationHandler *proxy,
    GCancellable *cancellable,
    GError **error);



/* ---- */

#define PHOSH_DBUS_TYPE_MOUNT_OPERATION_HANDLER_PROXY (phosh_dbus_mount_operation_handler_proxy_get_type ())
#define PHOSH_DBUS_MOUNT_OPERATION_HANDLER_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_MOUNT_OPERATION_HANDLER_PROXY, PhoshDBusMountOperationHandlerProxy))
#define PHOSH_DBUS_MOUNT_OPERATION_HANDLER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_MOUNT_OPERATION_HANDLER_PROXY, PhoshDBusMountOperationHandlerProxyClass))
#define PHOSH_DBUS_MOUNT_OPERATION_HANDLER_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_MOUNT_OPERATION_HANDLER_PROXY, PhoshDBusMountOperationHandlerProxyClass))
#define PHOSH_DBUS_IS_MOUNT_OPERATION_HANDLER_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_MOUNT_OPERATION_HANDLER_PROXY))
#define PHOSH_DBUS_IS_MOUNT_OPERATION_HANDLER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_MOUNT_OPERATION_HANDLER_PROXY))

typedef struct _PhoshDBusMountOperationHandlerProxy PhoshDBusMountOperationHandlerProxy;
typedef struct _PhoshDBusMountOperationHandlerProxyClass PhoshDBusMountOperationHandlerProxyClass;
typedef struct _PhoshDBusMountOperationHandlerProxyPrivate PhoshDBusMountOperationHandlerProxyPrivate;

struct _PhoshDBusMountOperationHandlerProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  PhoshDBusMountOperationHandlerProxyPrivate *priv;
};

struct _PhoshDBusMountOperationHandlerProxyClass
{
  GDBusProxyClass parent_class;
};

GType phosh_dbus_mount_operation_handler_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusMountOperationHandlerProxy, g_object_unref)
#endif

void phosh_dbus_mount_operation_handler_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusMountOperationHandler *phosh_dbus_mount_operation_handler_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusMountOperationHandler *phosh_dbus_mount_operation_handler_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void phosh_dbus_mount_operation_handler_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusMountOperationHandler *phosh_dbus_mount_operation_handler_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusMountOperationHandler *phosh_dbus_mount_operation_handler_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define PHOSH_DBUS_TYPE_MOUNT_OPERATION_HANDLER_SKELETON (phosh_dbus_mount_operation_handler_skeleton_get_type ())
#define PHOSH_DBUS_MOUNT_OPERATION_HANDLER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_MOUNT_OPERATION_HANDLER_SKELETON, PhoshDBusMountOperationHandlerSkeleton))
#define PHOSH_DBUS_MOUNT_OPERATION_HANDLER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_MOUNT_OPERATION_HANDLER_SKELETON, PhoshDBusMountOperationHandlerSkeletonClass))
#define PHOSH_DBUS_MOUNT_OPERATION_HANDLER_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_MOUNT_OPERATION_HANDLER_SKELETON, PhoshDBusMountOperationHandlerSkeletonClass))
#define PHOSH_DBUS_IS_MOUNT_OPERATION_HANDLER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_MOUNT_OPERATION_HANDLER_SKELETON))
#define PHOSH_DBUS_IS_MOUNT_OPERATION_HANDLER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_MOUNT_OPERATION_HANDLER_SKELETON))

typedef struct _PhoshDBusMountOperationHandlerSkeleton PhoshDBusMountOperationHandlerSkeleton;
typedef struct _PhoshDBusMountOperationHandlerSkeletonClass PhoshDBusMountOperationHandlerSkeletonClass;
typedef struct _PhoshDBusMountOperationHandlerSkeletonPrivate PhoshDBusMountOperationHandlerSkeletonPrivate;

struct _PhoshDBusMountOperationHandlerSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  PhoshDBusMountOperationHandlerSkeletonPrivate *priv;
};

struct _PhoshDBusMountOperationHandlerSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType phosh_dbus_mount_operation_handler_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusMountOperationHandlerSkeleton, g_object_unref)
#endif

PhoshDBusMountOperationHandler *phosh_dbus_mount_operation_handler_skeleton_new (void);


G_END_DECLS

#endif /* __PHOSH_GTK_MOUNTOPERATION_DBUS_H__ */
